/* Example of counter for DIO-64.

   Author: Reed Lai

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */


#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <signal.h>

#include "ixisa.h"

unsigned sig_counter;

void sig_handler(int sig)
{
	++sig_counter;
	printf("time beat %u\n", sig_counter);
}

int main()
{
	int fd;
	char *dev_file;
	ixisa_reg_t reg;
	ixisa_signal_t sig;

	static struct sigaction act, act_old;

	sig_counter = 0;
	dev_file = "/dev/ixisa1";

	/* open device file */
	fd = open(dev_file, O_RDWR);
	if (fd < 0) {
		printf("Failure of open device file \"%s.\"\n", dev_file);
		return FAILURE;
	}

	/* set action for signal */
	act.sa_handler = sig_handler;
	sigemptyset(&act.sa_mask);
	sigaddset(&act.sa_mask, SIGALRM);
	if (sigaction(SIGALRM, &act, &act_old)) {
		close(fd);
		puts("Failure of signal action.");
		return FAILURE;
	}

	/* set signal condiction for irq */
	sig.sid = SIGALRM;
	sig.pid = getpid();
	sig.is = 0x8;
		/* INT_CHAN_3 */
	sig.edge = 0x8;
		/* signal for negative edge */
	sig.bedge = 0;
		/* don't signal for both edges */

	if (ioctl(fd, IXISA_SET_SIG, &sig)) {
		close(fd);
		sigaction(SIGALRM, &act_old, NULL);
		puts("Failure of signal condiction.");
		return FAILURE;
	}
	/*****************************************
	 * setup 8254 counter-4/5 for INT_CHAN_3 *
	 *                                       *
	 * 32768Hz / (0x4000 * 2) = 1Hz          *
	 *****************************************/
	reg.id = IXISA_8254CR1;
	reg.value = 0x76;			/* counter-4, RL 3, mode 3, BCD 0*/
	if (ioctl(fd, IXISA_WRITE_REG, &reg)) {
		close(fd);
		sigaction(SIGALRM, &act_old, NULL);
		puts("Failure of 8254 counter-1 mode.");
		return FAILURE;
	}
	reg.id = IXISA_8254C4;
	reg.value = 0x0;
	if (ioctl(fd, IXISA_WRITE_REG, &reg)) {
		close(fd);
		sigaction(SIGALRM, &act_old, NULL);
		puts("Failure of 8254 counter 4 LSB.");
		return FAILURE;
	}
	reg.value = 0x40;
	if (ioctl(fd, IXISA_WRITE_REG, &reg)) {
		close(fd);
		sigaction(SIGALRM, &act_old, NULL);
		puts("Failure of 8254 counter 4 MSB.");
		return FAILURE;
	}
	reg.id = IXISA_8254CR1;
	reg.value = 0xb6;			/* counter-5, mode 3 */
	if (ioctl(fd, IXISA_WRITE_REG, &reg)) {
		close(fd);
		sigaction(SIGALRM, &act_old, NULL);
		puts("Failure of 8254 counter 5 mode.");
		return FAILURE;
	}
	reg.id = IXISA_8254C5;
	reg.value = 2;
	if (ioctl(fd, IXISA_WRITE_REG, &reg)) {
		close(fd);
		sigaction(SIGALRM, &act_old, NULL);
		puts("Failure of 8254 counter 5 LSB.");
		return FAILURE;
	}

	reg.value = 0;
	if (ioctl(fd, IXISA_WRITE_REG, &reg)) {
		close(fd);
		sigaction(SIGALRM, &act_old, NULL);
		puts("Failure of 8254 counter 5 MSB.");
		return FAILURE;
	}
	/* press <enter> to get out */

	puts("Press <enter> to exit");
	while (getchar() != 10);

	close(fd);
	sigaction(SIGALRM, &act_old, NULL);
	puts("End of program.");
	return SUCCESS;
}
