/* Example of interrupt handle for DIO-144.

   Author: Reed Lai

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */

/* File level history (record changes for this file here.)

   Enable the CN0PC3 interrupt.

   JP0
     +----------+
     |D   o o   |
     |          |
     |P   o o   |
     |  +-----+ |
     |E | o o | |
     |  +-----+ |
     +----------+

   Connect CN0 and CN3, or CN0PC3 and CN3PC3

   v 0.0.2  2 Dec 2002 by Reed Lai
     Improves signal operating.

   v 0.0.1 22 Jan 2002 by Reed Lai
     Correct the use instruction.

   v 0.0.0 18 Jan 2002 by Reed Lai
     Create, blah blah... */
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <signal.h>

#include "ixisa.h"

#define MYSIG 33

unsigned sig_counter;

void sig_handler(int sig)
{
	++sig_counter;
	putchar('+');
}

int main()
{
	int fd, i = 0;
	char *dev_file;
	ixisa_reg_t reg, reg2;
	ixisa_signal_t sig;

	static struct sigaction act, act_old;

	sig_counter = 0;
	dev_file = "/dev/ixisa1";

	/* open device file */
	fd = open(dev_file, O_RDWR);
	if (fd < 0) {
		printf("Failure of openning device file \"%s.\"\n", dev_file);
		return FAILURE;
	}

	/* signal action */
	act.sa_handler = sig_handler;
	sigemptyset(&act.sa_mask);
	sigaddset(&act.sa_mask, MYSIG);
	if (sigaction(MYSIG, &act, &act_old)) {
		close(fd);
		puts("Failure of signal action.");
		return FAILURE;
	}

	/* Configure CN0 ports as input */
	reg.id = IXISA_CN0CR;
//	reg.value = 0x9b;
 	reg.value = 0x99;
	if (ioctl(fd, IXISA_WRITE_REG, &reg)) {
		close(fd);
		sigaction(MYSIG, &act_old, NULL);
		puts("Failure of CN0.");
		return FAILURE;
	}
#if 0
	/* Configure CN3 ports as output */
	reg.id = IXISA_CN3CR;
	reg.value = 0x80;
	if (ioctl(fd, IXISA_WRITE_REG, &reg)) {
		close(fd);
		sigaction(MYSIG, &act_old, NULL);
		puts("Failure of CN3.");
		return FAILURE;
	}
#endif
	/* signal condiction */
	sig.sid = MYSIG;
	sig.pid = getpid();
	if (ioctl(fd, IXISA_SET_SIG, &sig)) {
		close(fd);
		sigaction(MYSIG, &act_old, NULL);
		puts("Failure of signal condition.");
		return FAILURE;
	}

	/* rocking time */
// 	reg.id = IXISA_CN3PC;
	reg.id = IXISA_CN0PB;
	reg2.id = IXISA_CN0PC;
	puts("press several space keys to IRQ, then enter to get out\n");
	while (i++ < 10) {
		reg.value = reg.value == 0xff ? 0 : 0xff;
		if (ioctl(fd, IXISA_WRITE_REG, &reg)) {
			close(fd);
			sigaction(MYSIG, &act_old, NULL);
			puts("Failure of rising.");
			return FAILURE;
		}
		usleep(50000);
		ioctl(fd, IXISA_READ_REG, &reg2);
		printf("%x ", reg2.value);
	}
	printf("\nGet interrupt signal for %d times.\nPush a key to get out: ", sig_counter);
	getchar();

	close(fd);
	sigaction(MYSIG, &act_old, NULL);
	puts("\nEnd of program.");
	return SUCCESS;
}
