/* Example of I/O port for DIO-96.

   Author: Reed Lai

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */


#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <unistd.h>

#include "ixisa.h"

int main()
{
	int fd;
	char *dev_file;
	ixisa_reg_t reg;
	int dio = 0;

	dev_file = "/dev/ixisa1";

	/* Open device file */
	fd = open(dev_file, O_RDWR);
	if (fd < 0) {
		printf("Failure of openning device file \"%s.\"\n", dev_file);
		return FAILURE;
	}

	/* Configure CN0 as output */
	reg.id = IXISA_CN0CR;
	reg.value = 0x80;
	if (ioctl(fd, IXISA_WRITE_REG, &reg)) {
		puts("Failure of writing register CN0CR!\n");
		close(fd);
		return FAILURE;
	}

	/* Configure CN3 as input */
	reg.id = IXISA_CN3CR;
	reg.value = 0x9b;
	if (ioctl(fd, IXISA_WRITE_REG, &reg)) {
		puts("Failure of writing register CN3CR!\n");
		close(fd);
		return FAILURE;
	}

	/* Write CN0 ports, read CN3 ports */
	while (getchar() == 10) {
		printf("write CN0P[ABC]: [");
		reg.id = IXISA_CN0PA;
		reg.value = dio++;
		if (!ioctl(fd, IXISA_WRITE_REG, &reg))
			printf("%2x", reg.value);
		else printf("--");
		
		reg.id = IXISA_CN0PB;
		reg.value = dio++;
		if (!ioctl(fd, IXISA_WRITE_REG, &reg))
			printf(" %2x", reg.value);
		else printf("--");

		reg.id = IXISA_CN0PC;
		reg.value = dio++;
		if (!ioctl(fd, IXISA_WRITE_REG, &reg))
			printf(" %2x]\n", reg.value);
		else printf("--]\n");

		printf("read  CN3P[ABC]: [");
		reg.id = IXISA_CN3PA;
		if (!ioctl(fd, IXISA_READ_REG, &reg))
			printf("%2x", reg.value);
		else printf("--");

		reg.id = IXISA_CN3PB;
		if (!ioctl(fd, IXISA_READ_REG, &reg))
			printf(" %2x", reg.value);
		else printf("--");

		reg.id = IXISA_CN3PC;
		if (!ioctl(fd, IXISA_READ_REG, &reg))
			printf(" %2x]\n", reg.value);
		else printf("--]\n");
		puts("Esc-enter to quit.");
	}

	close(fd);
	return SUCCESS;

}
