/* Read/write register by ioctl command IXISA_READ_REG and IXISA_WRITE_REG for
   digital I/O.

   Author: Reed Lai

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */

/* File level history (record changes for this file here.)

   v 0.0.0 11 Apr 2003 by Reed Lai
     I do this for Aiur! */

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <unistd.h>

#include "ixisa.h"
#include "_isold.h"

int main()
{
	int fd;
	char *dev_file;
	ixisa_reg_t sr, fifo;
	int i = 0, x=0, ok = 0, err = 0;
	
	sr.id = IXISA_SR;
	fifo.id = IXISA_FIFO;
	
	dev_file = "/dev/ixisa1";

	/* open device file */
	fd = open(dev_file, O_RDWR);
	if (fd < 0) {
		printf("Failure of open device file \"%s.\"\n", dev_file);
		return FAILURE;
	}

	/* Check Board */
	 ioctl(fd, IXISA_READ_REG, &sr);
	 while ((sr.value & 0x04) == 0) {
		 ioctl(fd, IXISA_READ_REG, &sr);
	 }
	 if ((sr.value & 0x04) != 0) {
		sr.value = CMD_WRITE_DI_MODE;
		ioctl(fd, IXISA_WRITE_REG, &sr);
		sr.value = 0x55;
		ioctl(fd, IXISA_WRITE_REG, &sr);

		ioctl(fd, IXISA_READ_REG, &sr);
		if ((sr.value & 0x04 != 0))
		{
			sr.value = CMD_READ_DI_MODE;
			ioctl(fd, IXISA_WRITE_REG, &sr);
			
			ioctl(fd, IXISA_READ_REG, &sr);
			while ((sr.value & 0x02) ==0)
			{
				ioctl(fd, IXISA_READ_REG, &sr);
			}
		
			ioctl(fd, IXISA_READ_REG, &fifo);
			if (fifo.value != 0x55)
				printf("CheckError, not 0x55, value = %x\n", fifo.value);		
		}else{
			printf("Read  error");
			return FAILURE;
		}
	 }else {
		printf("CheckBoard error\n");
		return FAILURE;
	 }
	 
	 ioctl(fd, IXISA_READ_REG, &sr);
	 while ((sr.value & 0x04) == 0) {
		 ioctl(fd, IXISA_READ_REG, &sr);
	 }
	 if ((sr.value & 0x04) != 0) {
		sr.value = CMD_WRITE_DI_MODE;
		ioctl(fd, IXISA_WRITE_REG, &sr);
		sr.value = 0xaa;
		ioctl(fd, IXISA_WRITE_REG, &sr);

		ioctl(fd, IXISA_READ_REG, &sr);
		if ((sr.value & 0x04 != 0))
		{
			sr.value = CMD_READ_DI_MODE;
			ioctl(fd, IXISA_WRITE_REG, &sr);
			
			ioctl(fd, IXISA_READ_REG, &sr);
			while ((sr.value & 0x02) ==0)
			{
				ioctl(fd, IXISA_READ_REG, &sr);
				if ((sr.value & 0x02) !=0) 
					break;
			}
		
			ioctl(fd, IXISA_READ_REG, &fifo);
			if (fifo.value != 0xaa){
				printf("CheckError, not 0xaa, value = %x\n", fifo.value);		
				return FAILURE;
			}
		}else{
			printf("sr error");
			return FAILURE;
		}
	 }else {
		printf("CheckBoard 0x04 error 0xaa, %x\n", i);
		return FAILURE;
	 }

	 ioctl(fd, IXISA_READ_REG, &sr);
	 while ((sr.value & 0x04) == 0) {
		 ioctl(fd, IXISA_READ_REG, &sr);
	 }
	
	 printf("please wait a second\n");
	 sr.value = CMD_WRITE_DI_MODE;
	 ioctl(fd, IXISA_WRITE_REG, &sr);
	 sr.value = 0xff;	/* All DI Active High */
	 ioctl(fd, IXISA_WRITE_REG, &sr);
	 for (i = 0; i < 128; i++)
	 {
		 ioctl(fd, IXISA_READ_REG, &sr);
		 while ((sr.value & 0x04) == 0) {
			 ioctl(fd, IXISA_READ_REG, &sr);
		 }
		 sr.value = CMD_WRITE_DO;
		 ioctl(fd, IXISA_WRITE_REG, &sr);
		 sr.value = i;
		 ioctl(fd, IXISA_WRITE_REG, &sr);
		 usleep(20000);
		 
		 ioctl(fd, IXISA_READ_REG, &sr);
		 while ((sr.value & 0x04) == 0) {
			 ioctl(fd, IXISA_READ_REG, &sr);
		 }
		 sr.value = CMD_READ_DI;
		 ioctl(fd, IXISA_WRITE_REG, &sr);
		 
		 ioctl(fd, IXISA_READ_REG, &sr);
		 while ((sr.value & 0x02) == 0) {
			 ioctl(fd, IXISA_READ_REG, &sr);
		 }
		 ioctl(fd, IXISA_READ_REG, &fifo);
		 if (i == fifo.value) 
			 ok++; 
		 else {
			 err++;
			 printf("(%x, %x)", i, fifo.value);
		 }
	 }
	 printf("\nok=%d, err=%d\n", ok, err);

	close(fd);
	return SUCCESS;
}

