/* Example of Digital I/O for P16R16DIO.

   Author: Reed Lai

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */

/* File level history (record changes for this file here.)

   v 0.0.0 18 Nov 2002 by Reed Lai
     create, blah blah... */

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <unistd.h>

#include "ixisa.h"

int main()
{
	int fd;
	char *dev_file;
	ixisa_reg_t ri, ro, rb;

	dev_file = "/dev/ixisa1";

	/* Open device file */
	fd = open(dev_file, O_RDWR);
	if (fd < 0) {
		printf("Failure of openning device file \"%s.\"\n", dev_file);
		return FAILURE;
	}

	/* DIO */
	ro.id = rb.id = IXISA_DO_A;
	ro.value = 1;
	ri.id = IXISA_DI_A;
	while (getchar() == 10) {
		(ro.value < 0x100) ? (ro.value <<= 1) : (ro.value = 1);
		if (ioctl(fd, IXISA_WRITE_REG, &ro)) {
			puts("Failure of write register IXISA_DO");
			break;
		}

		usleep(5000); /* 5ms for relay operating time */

		if (ioctl(fd, IXISA_READ_REG, &rb)) {
			puts("Failure of read register IXISA_DO");
			break;
		}

		if (ioctl(fd, IXISA_READ_REG, &ri)) {
			puts("Failure of read register IXISA_DI");
			break;
		}
		printf("Relay Output = 0x%2x\tRead Back = 0x%2x\tDI =0x%2x\tEsc to exit.\n", ro.value, rb.value, ri.value);
	}

	close(fd);
	return SUCCESS;
}
